mod client;
mod field;
mod queue;

use client::{Client, Reply, Request};
use field::Field;
use queue::Message;
use std::sync::{
    mpsc::{channel, Sender},
    Arc, RwLock,
};

#[derive(Clone)]
pub struct MoreThanText {
    registry: Arc<RwLock<Vec<Sender<Message>>>>,
    tx: Sender<Request>,
}

impl MoreThanText {
    pub fn new() -> Self {
        let tx = Client::start();
        Self {
            registry: Arc::new(RwLock::new(Vec::new())),
            tx: tx,
        }
    }

    pub fn request<F>(&self, _session: Option<F>) -> Reply
    where
        F: Into<Field>,
    {
        let (tx, rx) = channel();
        let req = Request::new(tx);
        self.tx.send(req).unwrap();
        rx.recv().unwrap()
    }
}
